package com.kingdee.eas.custom.shuiyou.task;



import cn.hutool.core.util.StrUtil;
import cn.hutool.cron.task.Task;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Maps;
import com.kingdee.bos.BOSException;
import com.kingdee.bos.Context;
import com.kingdee.eas.common.EASBizException;
import com.kingdee.eas.custom.shuiyou.RequestStateEnum;
import com.kingdee.eas.custom.shuiyou.RequestTypeEnum;
import com.kingdee.eas.custom.shuiyou.uitls.ISYUtilsFacade;

import com.kingdee.eas.custom.shuiyou.uitls.SYUCronTaskUtil;
import com.kingdee.eas.custom.shuiyou.uitls.SYUtilsFacadeFactory;
import com.kingdee.eas.util.app.DbUtil;
import com.kingdee.shr.base.syssetting.MSFServiceFacadeFactory;
import org.apache.log4j.Logger;

import java.io.Serializable;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.CountDownLatch;

public class TaskInfo extends AbstractTaskInfo implements Serializable, Task
{
    private static Logger logger =Logger.getLogger(TaskInfo.class);
    private StringBuilder msg=new StringBuilder();
    private boolean state=true;
    public static final String calculatingCode="100004";
    private int i=0;
    private Context ctx;
    private CountDownLatch cdl;
    public TaskInfo()
    {
        super();

    }
    protected TaskInfo(String pkField)
    {
        super(pkField);

    }
    public TaskInfo setContext(Context context)
    {
        this.ctx=context;
        return this;
    }

    public TaskInfo setCountDownLatch(CountDownLatch cdl)
    {
        this.cdl=cdl;
        return this;
    }
    /**
     * 当前任务状态
     * true 执行中
     * false 执行完
     * @return
     */
    public boolean isState() {
        return state;
    }

    public String getMsg() {
        return msg.toString();
    }

    @Override
    public void execute() {
        this.i=this.i+1;
        try {
            //设置执行中
            this.setRequestState(RequestStateEnum.Ongoing);
            ISYUtilsFacade syu = SYUtilsFacadeFactory.getLocalInstance(this.ctx);
            Map<String,String> map = Maps.newHashMap();
            map.put("requestId",this.getRequestId());
            //拼接参数
            if(StrUtil.isBlank(this.getParam())){
                JSONObject jsonObject = JSONUtil.parseObj(this.getParam());
                for(Map.Entry<String,Object> st : jsonObject.entrySet()){
                    Optional<Object> op = Optional.ofNullable(st.getValue());
                    map.put(st.getKey(),op.orElse("").toString());
                }
            }
            String getParam = JSONUtil.toJsonStr(map);
            logger.info("任务："+this.getId().toString()+",请求地址："+this.getUrl()+",请求参数："+getParam+"。");
            //调用远程接口
            String string = null;
            if(RequestTypeEnum.GET_VALUE.equals(this.getRequestType().getValue())){
                string = syu.get(this.getUrl(),getParam);
            }else{
                string = syu.post(this.getUrl(),getParam);
            }

            JSONObject jsonObject = JSONUtil.parseObj(string);
            JSONObject head =  jsonObject.getJSONObject("head");
            if(null!=head){
                String state = (String) head.get("state");
                String code = (String) head.get("code");
                if("Y".equalsIgnoreCase(state)){
                    logger.info("请求成功"+(this.i)+",结果："+string);
                    //请求成功停止这个任务
                    this.stop();
                    Map<String,Object> param = JSONUtil.parseObj(this.getParam());
                    param.put("result",string);
                    this.setBody(JSONUtil.toJsonStr(param));
                    //设置成功
                    this.setRequestState(RequestStateEnum.Successful);
                    this.callBackOSF(this,param);
                } else if(calculatingCode.equals(code)){
                    logger.info("请求"+(this.i)+"计算中,结果："+string);
                    //继续
                } else {
                    logger.info("请求"+(this.i)+"失败,结果："+string);
                    //请求失败停止这个任务
                    this.stop();
                    Map<String,Object> param = JSONUtil.parseObj(this.getParam());
                    param.put("result",string);
                    this.setBody(JSONUtil.toJsonStr(param));
                    //设置失败
                    this.setRequestState(RequestStateEnum.Failed);
                    this.callBackOSF(this,param);
                }
            }
        }catch (BOSException e){
            e.printStackTrace();
            logger.error(e);
            this.stop();
        }finally {

        }
    }

    public void callBackOSF(TaskInfo task,Map<String,Object> param){
        try {
            //回调osf服务
            if(StrUtil.isNotBlank(this.getOsfServiceName())) {
                Object obj = MSFServiceFacadeFactory.getLocalInstance(this.ctx).processService(this.getOsfServiceName(), param);
            }
            //更新任务状态
            TaskFactory.getLocalInstance(this.ctx).save(this);
        } catch (BOSException e){
            //e.printStackTrace();
            logger.error(e);
        } catch (EASBizException e){
            //e.printStackTrace();
            logger.error(e);
        }finally {
            try {
                DbUtil.execute(ctx, "update CT_TK_Task set CFRequestState=? where fid=?",
                        new String[]{task.getRequestState().getValue(),task.getId().toString()});
            }catch (Exception e){
                logger.error(e);
            }
        }
    }

    public boolean stop(){
        this.state=false;
        if(null!=cdl) {
            cdl.countDown();
        }
        return SYUCronTaskUtil.getInstance().delSchedule(this);
    }
}